#!/usr/bin/python3
"""
## © University of Southampton IT Innovation Centre, 2018
##
## Copyright in this software belongs to University of Southampton
## IT Innovation Centre of Gamma House, Enterprise Road,
## Chilworth Science Park, Southampton, SO16 7NS, UK.
##
## This software may not be used, sold, licensed, transferred, copied
## or reproduced in whole or in part in any manner or form or in or
## on any media by any person other than in accordance with the terms
## of the Licence Agreement supplied with the software, or otherwise
## without the prior written consent of the copyright owners.
##
## This software is distributed WITHOUT ANY WARRANTY, without even the
## implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
## PURPOSE, except where stated in the Licence Agreement supplied with
## the software.
##
##      Created By :            Nikolay Stanchev
##      Created Date :          04-06-2018
##      Created for Project :   FLAME
"""


from clmcservice.utilities import generate_e2e_delay_report

"""
A python module which provides auxiliary functions to mimic the behaviour of an InfluxDBClient when unit testing the aggregator.
"""


class MockResultSet(object):
    """
    A mock object used to mimic the behaviour of a ResultSet in the influx library (we only need the functionality of an object that collects
    a group of points and has an items() method to get the collected points.
    """

    def __init__(self, points):
        """
        Initialise the mock result set.

        :param points: the collected points
        """

        self.points = points

    def items(self):
        """
        Get the data points in the result set.

        :return: the collected data points
        """

        return self.points


# The following are network-related auxiliary functions to generate test data.

def _network_result_point(net_latency, net_bandwidth):
    """
    Returns a generator, which yields one data point representing a network measurement (fields only)

    :param net_latency: the reported network latency
    :param net_bandwidth: the reported network bandwidth.

    :return: a generator object with one element (same behaviour is used in the influxdb library even when only one point is returned from the query)
    """

    yield {"net_latency": net_latency, "net_bandwidth": net_bandwidth}


def _network_tags(path, source, target):
    """
    Returns a dictionary representing a network measurement (tags only)

    :param path: the path identifier
    :param source: the source service router
    :param target: the target service router

    :return: a dictionary with those values
    """

    return {"path": path, "source": source, "target": target}


def _network_metadata(measurement, path, source, target):
    """
    Returns an influxdb-styled metadata about a network measurement.

    :param measurement: the measurement table name
    :param path: the path identifier
    :param source: the source service router
    :param target: the target service router

    :return: a tuple with the first element being the measurement name and the second element being a dictionary with the network measurement tag values
    """

    return measurement, _network_tags(path, source, target)


def network_result_item(measurement, path, source, target, net_latency, net_bandwidth):
    """
    Returns a full influxdb-styled network measurement item - with tag and field values.

    :param measurement: the measurement table name
    :param path: the path identifier
    :param source: the source service router
    :param target: the target service router
    :param net_latency: the reported network latency
    :param net_bandwidth: the reported network bandwidth.

    :return: a tuple with the first element being the result metadata (measurement name and tags) and the second element being the data field points
    """

    return _network_metadata(measurement, path, source, target), _network_result_point(net_latency, net_bandwidth)


# The following are service-related auxiliary functions to generate test data.

def _service_result_point(response_time, request_size, response_size):
    """
    Returns a generator, which yields one data point representing a service measurement (fields only)

    :param response_time: the response time of the service
    :param request_size: the averaged request size of the service
    :param response_size: the averaged response size of the service

    :return: a generator object with one element (same behaviour is used in the influxdb library even when only one point is returned from the query)
    """

    yield {"response_time": response_time, "request_size": request_size, "response_size": response_size}


def _service_tags(sfr, endpoint, sf_instance):
    """
    Returns a dictionary representing a service measurement (tags only)

    :param sfr: the service router to which the service's endpoint is connected to
    :param endpoint: the endpoint the service is being deployed on
    :param sf_instance: the service function instance (FQDN)

    :return: a dictionary with those values
    """

    return {"sfr": sfr,  "endpoint": endpoint, "sf_instance": sf_instance}


def _service_metadata(measurement, sfr, endpoint, sf_instance):
    """
    Returns an influxdb-styled metadata about a service measurement.

    :param measurement: the measurement table name
    :param sfr: the service router to which the service's endpoint is connected to
    :param endpoint: the endpoint the service is being deployed on
    :param sf_instance: the service function instance (FQDN)

    :return: a tuple with the first element being the measurement name and the second element being a dictionary with the service measurement tag values
    """

    return measurement, _service_tags(sfr, endpoint, sf_instance)


def service_result_item(measurement, sfr, endpoint, sf_instance, response_time, request_size, response_size):
    """
    Returns a full influxdb-styled service measurement item - with tag and field values.

    :param measurement: the measurement table name
    :param sfr: the service router to which the service's endpoint is connected to
    :param endpoint: the endpoint the service is being deployed on
    :param sf_instance: the service function instance (FQDN)
    :param response_time: the response time of the service
    :param request_size: the averaged request size of the service
    :param response_size: the averaged response size of the service

    :return: a tuple with the first element being the result metadata (measurement name and tags) and the second element being the data field points
    """

    return _service_metadata(measurement, sfr, endpoint, sf_instance), _service_result_point(response_time, request_size, response_size)


# The following are auxiliary functions for generating an e2e row used in the unit testing of the aggregator.

def drop_timestamp(d):
    """
    Drops the time stamp from a dictionary-represented influx result item object

    :param d: the dictionary object representing a measurement row from influx

    :return: the same dictionary with no timestamp
    """

    d.pop('time')
    return d


def _generate_e2e_row(path_id, source_sfr, target_sfr, endpoint, sf_instance, delay_forward, delay_reverse, delay_service, avg_request_size, avg_response_size, avg_bandwidth):
    """
    Generates a combined averaged measurement about the e2e delay and its contributing parts with default timestamp (set as 0)

    :param path_id: The path identifier, which is a bidirectional path ID for the request and the response path
    :param source_sfr: source service router
    :param target_sfr: target service router
    :param endpoint: endpoint of the media component
    :param sf_instance: service function instance (media component)
    :param delay_forward: Path delay (Forward direction)
    :param delay_reverse: Path delay (Reverse direction)
    :param delay_service: the media service component response time
    :param avg_request_size: averaged request size
    :param avg_response_size: averaged response size
    :param avg_bandwidth: averaged bandwidth

    :return: a list of dict-formatted reports to post on influx
    """

    return generate_e2e_delay_report(path_id, source_sfr, target_sfr, endpoint, sf_instance, delay_forward,
                                     delay_reverse, delay_service, avg_request_size, avg_response_size, avg_bandwidth, 0)[0]


def generate_e2e_no_timestamp_row(path_id, source_sfr, target_sfr, endpoint, sf_instance, delay_forward, delay_reverse,
                                  delay_service, avg_request_size, avg_response_size, avg_bandwidth):
    """
    Generates a combined averaged measurement about the e2e delay and its contributing parts (with no timestamp, used for testing)

    :param path_id: The path identifier, which is a bidirectional path ID for the request and the response path
    :param source_sfr: source service router
    :param target_sfr: target service router
    :param endpoint: endpoint of the media component
    :param sf_instance: service function instance (media component)
    :param delay_forward: Path delay (Forward direction)
    :param delay_reverse: Path delay (Reverse direction)
    :param delay_service: the media service component response time
    :param avg_request_size: averaged request size
    :param avg_response_size: averaged response size
    :param avg_bandwidth: averaged bandwidth

    :return: a list of dict-formatted reports to post on influx
    """

    return drop_timestamp(_generate_e2e_row(path_id, source_sfr, target_sfr, endpoint, sf_instance, delay_forward, delay_reverse, delay_service,
                                             avg_request_size, avg_response_size, avg_bandwidth))
